#include "udf.h"

//real limiter = 1e-10;

real fmax(real x, real y)
{
	if(x > y)
		return x;
	else
		return y;
}

real fmin(real x, real y)
{
	if(x < y)
		return x;
	else return y;
}

DEFINE_INIT(wall_dist, d)
{
	cell_t c;
	Thread *t;
	real xc[ND_ND];
	real x, y, WD, D;
	
	D = 0.02;
	
	thread_loop_c(t,d)
	{
		begin_c_loop_all(c,t)
		{
			C_CENTROID(xc,c,t);
			
			x = xc[0];
			y = xc[1];
			
			if(x<=0) WD = 0.01-y;
			else if(x>0 && x*x+(y-0.01)*(y-0.01)<=(x-0.04)*(x-0.04))
				WD = sqrt(x*x+(y-0.01)*(y-0.01));
			else WD = 0.04-x;
			
			C_UDMI(c,t,1) = WD;
		}
		end_c_loop_all(c,t)
	}
}

DEFINE_PROPERTY(turbulent_viscosity, c, t)
{
	real x[ND_ND];
	real mut, mu, nu, rho, k, omega, a1, arg2, y, OmF2, Omega, ux, uy, vx, vy, s11, s12, s21, s22;
	
	mu = 1.7894e-5;
	rho = 1.225;
	a1 = 0.31;
	nu = mu/rho;
	
	k = fmax(C_UDSI(c,t,0),1e-10);
	omega = fmax(C_UDSI(c,t,1),1e-10);
	y = C_UDMI(c,t,1); // wall distance
	
	ux = C_U_G(c,t)[0];
	uy = C_U_G(c,t)[1];
	vx = C_V_G(c,t)[0];
	vy = C_V_G(c,t)[1];
	
	s11 = ux;
	s12 = 0.5*(uy+vx);
	s21 = s12;
	s22 = vy;
	
	Omega = sqrt(2.0*(s11*s11+s12*s12+s21*s21+s22*s22));
	
	arg2 = fmax(2*sqrt(k)/(0.09*omega*y), 500*nu/(y*y*omega));
	OmF2 = Omega*tanh(arg2*arg2);
	
	mut = rho*a1*k/fmax(a1*omega,OmF2);
	C_UDMI(c,t,0) = mut;
	
	return mu+mut;
}

DEFINE_SOURCE(k_production,c,t,dS,eqn)
{
	real rho;
	real omega, k, Gk, mut, ux, uy, vx, vy, s11, s12, s21, s22, betastar;
	
	rho = 1.225;
	betastar = 0.09;
	
	// Policz gradienty
	ux = C_U_G(c,t)[0];
	uy = C_U_G(c,t)[1];
	vx = C_V_G(c,t)[0];
	vy = C_V_G(c,t)[1];
	
	s11 = ux;
	s12 = 0.5*(uy+vx);
	s21 = s12;
	s22 = vy;
	
	mut = C_UDMI(c,t,0);
	Gk = mut*2.0*(s11*s11+s12*s12+s21*s21+s22*s22);
	k = C_UDSI(c,t,0);
	omega = C_UDSI(c,t,1);
	
	//return fmin(Gk, 10*rho*betastar*k*omega) - betastar*rho*omega*k;
	return fmin(Gk, 10*rho*betastar*k*omega) - betastar*rho*omega*k;
}

DEFINE_PROPERTY(k_diffusivity, c, t)
{
	real sigmak, sigmak1, sigmak2, F1, arg1;
	real mu, mut, k, omega, y, nu, rho, sigmaomega2, CD, dkdw, dkx1, dkx2, dwx1, dwx2;
	
	mu = 1.7984e-5;
	
	k = fmax(C_UDSI(c,t,0),1e-10);
	omega = fmax(C_UDSI(c,t,1),1e-10);
	y = C_UDMI(c,t,1);
	rho = 1.225;
	nu = mu/rho;
	sigmaomega2 = 0.856;
	sigmak1 = 0.5;
	sigmak2 = 1.0;
	dkx1 = C_UDSI_G(c,t,0)[0];
	dkx2 = C_UDSI_G(c,t,0)[1];
	dwx1 = C_UDSI_G(c,t,1)[0];
	dwx2 = C_UDSI_G(c,t,1)[1];
	dkdw = dkx1*dwx1+dkx2*dwx2;
	CD = fmax(2*rho*sigmaomega2*dkdw/omega,1e-10);
	
	arg1 = fmin(fmax(sqrt(k)/(0.09*omega*y),500*nu/(y*y*omega)),4*rho*sigmaomega2*k/(CD*y*y));
	F1 = tanh(arg1*arg1*arg1*arg1);
	sigmak = F1*sigmak1+(1-F1)*sigmak2;
	mut = C_UDMI(c,t,0);
	C_UDMI(c,t,2) = F1;
	C_UDMI(c,t,3) = mu+mut*sigmak;
	
	return mu+mut*sigmak;
	/*if(c == 0) printf("%f ", mu+mut*sigmak); */
	//return (real)2;
}

DEFINE_SOURCE(omega_production,c,t,dS,eqn)
{
	real rho;
	real omega, k, Gk, mu, mut, ux, uy, vx, vy, s11, s12, s21, s22, betastar, y, nu, CD, dkdw, dkx1, dkx2, dwx1, dwx2;
	real gamma1, gamma2, beta1, beta2, sigmaomega1, sigmaomega2, kappa, gamma, beta, nut, F1, arg1;
	real Term3;
	
	rho = 1.225;
	mu = 1.7984e-5;
	betastar = 0.09;
	beta1 = 0.075;
	beta2 = 0.0828;
	sigmaomega1 = 0.5;
	sigmaomega2 = 0.856;
	kappa = 0.41;
	gamma1 = beta1/betastar-sigmaomega1*kappa*kappa/sqrt(betastar);
	gamma2 = beta2/betastar-sigmaomega2*kappa*kappa/sqrt(betastar);
	
	// Policz gradienty
	ux = C_U_G(c,t)[0];
	uy = C_U_G(c,t)[1];
	vx = C_V_G(c,t)[0];
	vy = C_V_G(c,t)[1];
	
	s11 = ux;
	s12 = 0.5*(uy+vx);
	s21 = s12;
	s22 = vy;
	
	mut = C_UDMI(c,t,0);
	Gk = mut*2.0*(s11*s11+s12*s12+s21*s21+s22*s22);
	k = fmax(C_UDSI(c,t,0),1e-10);
	omega = fmax(C_UDSI(c,t,1),1e-10);
	
	dkx1 = C_UDSI_G(c,t,0)[0];
	dkx2 = C_UDSI_G(c,t,0)[1];
	dwx1 = C_UDSI_G(c,t,1)[0];
	dwx2 = C_UDSI_G(c,t,1)[1];
	dkdw = dkx1*dwx1+dkx2*dwx2;
	CD = fmax(2*rho*sigmaomega2*dkdw/omega,1e-20);
	
	y = C_UDMI(c,t,1);
	nu = mu/rho;
	
	arg1 = fmin(fmax(sqrt(k)/(0.09*omega*y),500*nu/(y*y*omega)),4*rho*sigmaomega2*k/(CD*y*y));
	F1 = tanh(arg1*arg1*arg1*arg1);
	
	//C_UDMI(c,t,2) = F1;
	
	nut = mut/rho;
	gamma = F1*gamma1+(1-F1)*gamma2;
	beta = F1*beta1+(1-F1)*beta2;
	
	// Trzeci czlon
	Term3 = 2*rho*(1-F1)*sigmaomega2*dkdw/omega;
	
	return gamma/nut*fmin(Gk, 10*rho*betastar*k*omega) - beta*rho*omega*omega + Term3;
}

DEFINE_PROPERTY(omega_diffusivity, c, t)
{
	real mu, mut, sigmaomega, sigmaomega1, sigmaomega2, F1, arg1, k, omega, y, nu, rho, CD;
	real dkdw, dkx1, dkx2, dwx1, dwx2;
	
	mu = 1.7984e-5;
	
	k = fmax(C_UDSI(c,t,0),1e-10);
	omega = fmax(C_UDSI(c,t,1),1e-10);
	y = C_UDMI(c,t,1);
	rho = 1.225;
	nu = mu/rho;
	sigmaomega1 = 0.5;
	sigmaomega2 = 0.856;
	dkx1 = C_UDSI_G(c,t,0)[0];
	dkx2 = C_UDSI_G(c,t,0)[1];
	dwx1 = C_UDSI_G(c,t,1)[0];
	dwx2 = C_UDSI_G(c,t,1)[1];
	dkdw = dkx1*dwx1+dkx2*dwx2;
	CD = fmax(2*rho*sigmaomega2*dkdw/omega,1e-20);
	
	arg1 = fmin(fmax(sqrt(k)/(0.09*omega*y),500*nu/(y*y*omega)),4*rho*sigmaomega2*k/(CD*y*y));
	F1 = tanh(arg1*arg1*arg1*arg1);
	sigmaomega = F1*sigmaomega1+(1-F1)*sigmaomega2;
	mut = C_UDMI(c,t,0);
	
	return mu+mut*sigmaomega;
	//return (real)3.;
}

DEFINE_PROFILE(omega_bc, t, i)
{
	cell_t c0;
	real x[ND_ND];
	real y, beta1, nu;
	face_t f;
	
	nu = 1.7894e-5;
	beta1 = 0.075;
	
	begin_f_loop(f,t)
	{
		// Znajdz numer komorki
		c0 = F_C0(f,t);
		
		y = C_UDMI(c0,THREAD_T0(t), 1);
		
		F_PROFILE(f,t,i) = 10*6*nu/(beta1*y*y);
	}
	end_f_loop(f, t);
}
